###############################################################
###############################################################
#                      Short course		              #
# 		"Basketball Data Analysis"	              #
###############################################################
###############################################################

rm(list=ls())
# install.packages("devtools", repos="https://cran.stat.unipd.it/")
# devtools::install_github("sndmrc/BasketballAnalyzeR",force=TRUE)
library(BasketballAnalyzeR)

# setwd("......")

load("NBA1819.RData")
Obox <- Obox1819
Pbox <- Pbox1819
Tadd <- Tadd1819
Tbox <- Tbox1819

load("PbP_BDB_CLE.Rdata")
PbP <- PbPmanipulation(PbP.BDB.CLE, playTeam="CLE")

######################
# 2.2.1 Pace, Ratings, Four Factors
######################
#Q1: What were offensive and defensive performance of the four NBA Conference finalists?
tm <- c("MIL","TOR","GSW","POR")
selTeams <- which(Tadd$team %in% tm)
FF.sel <- fourfactors(Tbox[selTeams,], Obox[selTeams,])

#Figure
plot(FF.sel)

######################
# 2.2.2 Bar-line plots
######################
#rm(list=ls())
#Q2: Which NBA Easter conference team had the best performance according to defensive statistics?
X <- data.frame(Tbox, PTS.Opp=Obox$PTS,TOV.O=Obox$TOV,
				CONF=Tadd$Conference)
XW <- subset(X, CONF=="E")
labs <- c("Steals","Blocks","Defensive Rebounds")

#Figure 
barline(data=XW, id="Team", bars=c("STL","BLK","DREB"),
		line="TOV.O", order.by="PTS.Opp", labels.bars=labs)
		 
#####################
# 2.2.3 Radial plots
#####################
#rm(list=ls())
#Q3: Can I compare the game profiles of the three best centers in NBA 2018/19?
Pbox.PG <- subset(Pbox, Player=="Anthony Davis" |
				  Player=="Joel Embiid" |
				  Player=="Karl-Anthony Towns" )
attach(Pbox.PG)
X <- data.frame(P2M, P3M, FTM, REB=OREB+DREB, AST,
				STL, BLK)/MIN
detach(Pbox.PG)

#Figure
pdf(file="radialplot1.pdf",width=9,height=3,paper="special")
radialprofile(data=X, title=Pbox.PG$Player, std=FALSE, ncol.arrange=3)
dev.off()

#Figure
pdf(file="radialplot2.pdf",width=9,height=3,paper="special")
radialprofile(data=X, title=Pbox.PG$Player, std=TRUE, ncol.arrange=3)  
dev.off()


#####################
# 2.2.5 Bubble plots
#####################
#rm(list=ls())
#Q4: Which team had the best shooting performance in NBA 2018/19?

attach(Tbox)
X <- data.frame(T=Team, P2p, P3p, FTp, AS=P2A+P3A+FTA)
detach(Tbox)
labs <- c("2-point shots (% made)",
		  "3-point shots (% made)",
		  "free throws (% made)",
		  "Total shots attempted")

#Figure 5
bubbleplot(X, id="T", x="P2p", y="P3p", col="FTp",
		   size="AS", labels=labs)

#Q5a: Which players of the two NBA finalists had the best shooting performance in NBA 2018/19?
Pbox.GSW.TR <- subset(Pbox,
			   (Team=="Golden State Warriors" |
					  Team =="Toronto Raptors") &
					  MIN>=1000)
attach(Pbox.GSW.TR)
X <- data.frame(ID=Player, Team, P2p, P3p, FTp, AS=P2A+P3A+FTA)
detach(Pbox.GSW.TR)
labs <- c("2-point shots (% made)",
		  "3-point shots (% made)",
		  "free throws (% made)",
		  "Total shots attempted")

#Figure
bubbleplot(X, id="ID", x="P2p", y="P3p", col="FTp",
		   size="AS", text.col="Team", labels=labs,
		   title="Golden State Warriors and Toronto Raptors during the regular season",
		   text.legend=TRUE, text.size=3.5, scale=FALSE)


#Q5b: Which players of the two NBA finalists showed the best defense?
Pbox.GSW.CC <- subset(Pbox,
			   (Team=="Golden State Warriors" |
					  Team =="Toronto Raptors") &
					  MIN>=1000)
attach(Pbox.GSW.CC)
X <- data.frame(ID=Player, Team, V1=DREB/MIN, V2=STL/MIN,
				V3=BLK/MIN, V4=MIN)
detach(Pbox.GSW.CC)
labs <- c("Defensive Rebounds","Steals","Blocks",
		  "Total minutes played")
		  
#Figure
bubbleplot(X, id="ID", x="V1", y="V2", col="V3",
		   size="V4", text.col="Team", labels=labs,
		   title="Golden State Warriors and Toronto Raptors during the regular season",
		   text.legend=TRUE, text.size=3.5, scale=FALSE)



######################
# 2.2.6 Variability analysis
######################
#rm(list=ls())
#Q6: How different is the performance on the 3-point shots among the players of Los Angeles Lakers?

Pbox.OKC <- subset(Pbox, Team=="Los Angeles Lakers"
				   & MIN>=500 & P3A>1)
				   
tabella <- cbind(Pbox.OKC$Player, round(Pbox.OKC$P3p,2),Pbox.OKC$P3A)
tabellaord <- tabella[order(-Pbox.OKC$P3p),]
library(xtable)
xtable(tabellaord)
#vrb1 <- variability(data=Pbox.OKC, data.var="P3p",
					#size.var="P3A")

vrb1 <- variability(data=Pbox.OKC, data.var="P3p",
					size.var="P3A",weight=FALSE)
mean(vrb1$data[,1])

vrb2 <- variability(data=Pbox.OKC, data.var="P3p",
					size.var="P3A",weight=TRUE)

#vrb2 <- variability(data=Pbox.OKC,
					#data.var=c("P2p","P3p","FTp"),
					#size.var=c("P2A","P3A","FTA"),
					#weight=TRUE)

#Figure
plot(vrb2, title="Variability of 3-point shot percentage - Los Angeles Lakers")


######################
# 2.2.7 Inequality analysis
######################
#rm(list=ls())
#Q7: How balanced is the San Antonio Spurs 3-point shot performance?

no.teams <- nrow(Tbox)
INEQ <- array(0, no.teams)
for (k in 1:no.teams) {
	 Teamk <- Tbox$Team[k]
	 Pbox.sel <- subset(Pbox, Team==Teamk)
	 index <- inequality(Pbox.sel$P3M, npl=8)
	 INEQ[k] <- index$Gini
	 }

dts <- data.frame(INEQ, P3M=Tbox$P3M,
				  CONF=Tadd$Conference)
mypal <- colorRampPalette(c("blue","red"))

#Figure
scatterplot(dts, data.var=c("INEQ","P3M"), z.var="CONF",
			labels=Tbox$Team, palette=mypal,
			repel_labels=TRUE)
cbind(T, INEQ)

k<- 27
Pbox.sel <- subset(Pbox, Team==T[k])
AA <- cbind(Pbox.sel$Player, Pbox.sel$P3M,Pbox.sel$P3A)
AAord <- AA[order(-Pbox.sel$P3M),]
AAord <- AAord[1:8,]
xtable(AAord)

round(cumsum(AAord[,2])/cumsum(AAord[,2])[8]*100,2)

k<- 11
Pbox.sel <- subset(Pbox, Team==T[k])
AA <- cbind(Pbox.sel$Player, Pbox.sel$P3M,Pbox.sel$P3A)
AAord <- AA[order(-Pbox.sel$P3M),]
AAord <- AAord[1:8,]

round(cumsum(AAord[,2])/cumsum(AAord[,2])[nrow(AAord)]*100,2)
#####################
# 2.2.8 Shot charts
#####################
#rm(list=ls())
#Q8: What are LeBron James's favorite and disliked spots on the court?

subdata <- subset(PbP, player=="LeBron James")
subdata$xx <- subdata$original_x/10
subdata$yy <- subdata$original_y/10-41.75

subdata$xx <- subdata$original_x/10
subdata$yy <- subdata$original_y/10-41.75

shotchart(data=subdata, x="xx", y="yy", type=NULL,
		  scatter=TRUE)

#Figure left
shotchart(data=subdata, x="xx", y="yy", z="result", type=NULL,
		  scatter=TRUE)

shotchart(data=subdata, x="xx", y="yy", z="playlength", 
          num.sect=5, type="sectors", scatter = TRUE)

#Figure right
shotchart(data=subdata, x="xx", y="yy", z="playlength", 
       	  num.sect=5, type="sectors", scatter=FALSE, result="result")

#####################
# 3.4 ANALYZING NETWORK RELATIONSHIPS
#####################
#Q9: How does the network of assists work in the team of Cleveland Cavaliers?
PbP.GSW <- subset(PbP, team=="CLE")

netdata <- assistnet(PbP.GSW)
netdata
set.seed(7)

#Figure left
plot(netdata)

#Figure right
plot(netdata, layout="circle", edge.thr=20,
	 node.col="FGPTS_AST", node.size="ASTPTS")

#####################
# 3.5.1  Density with respect to a concurrent variable
#####################
#Q10: How does the frequency of 2-point shots by Cleveland Cavaliers vary during the match?
#rm(list=ls())


data.team <- subset(PbP, team=="CLE" & result!="")
data.opp <- subset(PbP, team!="CLE" & result!="")

#Figure left
densityplot(data=data.team, shot.type="2P",
			var="periodTime", best.scorer=TRUE)
			
#Figure right			
densityplot(data=data.team, shot.type="2P",
			var="totalTime", best.scorer=TRUE)


#######################
# 5.2.2.2 Estimation of expected points
#######################
#Q11: Which is the best distance to shoot from for LeBron James? And for his teammates?
rm(list=ls())
load("PbP_BDB_CLE.Rdata")
PbP <- PbPmanipulation(PbP.BDB.CLE, playTeam="CLE")


PbP.GSW <- subset(PbP, team=="CLE")
pl <- c("LeBron James")
mypal <- colorRampPalette(c("red"))

# Figure left
expectedpts(data=PbP.GSW, players=pl,
			col.team="gray", palette=mypal,
			col.hline="gray")


Pbox.GSW <- subset(Pbox, PTS>=500 &
				   Team=="Cleveland Cavaliers")			   
				   				   
pl <- Pbox.GSW$Player
mypal <- colorRampPalette(c("green","blue"))

# Figure right
expectedpts(data=PbP, players=pl,
			col.team="gray", palette=mypal,
			col.hline="gray")